with Ada.Text_IO; -- for our usual "printf" (Put[_Line])
with Museum;

package body Tourists is

   --  we need a modular type that wraps around
   --  in order for task instances to keep count of
   --  their infinite number of visits to the Museum
   Limit : constant Positive := 10;
   type Counter_T is mod Limit;

   --+---------------------------
   task body Visitor_T is
      use Museum,
          Ada.Text_IO;

      --  each process instance will keep a running
      --  counter of the number of visits paid to
      --  the very beautiful museum guarded by
      --  Museum.Guardian
      --+
      --  Visit never overflows but just wraps around at Limit
      Visit : Counter_T := 0;

   begin
      loop
         Visit := Visit + 1;
         Put_Line
           ("Visitor #" & Positive'Image (Id)
            & " queuing for visit nr. "
            & Counter_T'Image (Visit)
            & " to the Museum.");

         --  the process applies for entrance
         Guardian.Entrance;
         --  permission granted

         --  a relative-duration visit
         delay Duration (Interval);

         --  the process notifies exit
         Guardian.Sortie;

         Put_Line
           ("Visitor #" & Positive'Image (Id)
            & " was granted permission to enter.");
         --  at this point, the process sleeps,
         --  dines and then repeats the whole cycle
         delay Duration (Interval * 2);
      end loop;
   end Visitor_T;

end Tourists;
